@@ -1,4 +1,5 @@
 import os
+from operator import attrgetter
 from typing import List, TextIO, Dict, AnyStr, Sequence, Tuple, Union
 import logging
 
@@ -155,7 +156,7 @@ class Simulation:
             # POST INIT (post read)
             self.out_file_path = os.path.join(self.root_path, self.chid + ".out")
             self.ventilations: List[Ventilation] = list(self.ventilations.values())
-            self.obstructions = ObstructionCollection(self.obstructions.values())
+            self.obstructions = ObstructionCollection(self._merged_obstructions)
             for obst in self.obstructions:
                 obst._post_init()
             # Combine the gathered temporary information into data collections
@@ -227,17 +228,25 @@ class Simulation:
     def _load_obstructions(self, smv_file: TextIO, mesh: Mesh):
         temp_data = list()
         n = int(smv_file.readline().strip())
+
+        if n != 0:
+            self.obstructions[mesh] = list()
+
         for _ in range(n):
             line = smv_file.readline().strip().split()
             ext = [float(line[i]) for i in range(6)]
-            obst_index = int(line[6]) - 1
+            obst_ordinal = int(line[6])
 
             side_surfaces = tuple(self.surfaces[int(line[i]) - 1] for i in range(7, 13))
             if len(line) > 13:
                 texture_origin = (float(line[13]), float(line[14]), float(line[15]))
-                temp_data.append((obst_index, Extent(*ext), side_surfaces, texture_origin))
+                temp_data.append((obst_ordinal, Extent(*ext), side_surfaces, texture_origin))
             else:
-                temp_data.append((obst_index, Extent(*ext), side_surfaces))
+                temp_data.append((obst_ordinal, Extent(*ext), side_surfaces))
+
+        # We need to save the extents of obstructions that have been created due to a hole to later
+        # recreate a Hole with the correct extents
+        hole_extents: Dict[int, List[Extent]] = dict()
 
         for tmp in temp_data:
             line = smv_file.readline().strip().split()
@@ -252,17 +261,59 @@ class Simulation:
                 rgba = ()
 
             if len(tmp) == 4:
-                obst_index, extent, side_surfaces, texture_origin = tmp
+                obst_ordinal, extent, side_surfaces, texture_origin = tmp
             else:
-                obst_index, extent, side_surfaces = tmp
+                obst_ordinal, extent, side_surfaces = tmp
                 texture_origin = self.default_texture_origin
 
-            if obst_index not in self.obstructions:
-                self.obstructions[obst_index] = Obstruction(obst_index, side_surfaces,
-                                                            bound_indices, color_index, block_type,
-                                                            texture_origin, rgba=rgba)
-            self.obstructions[obst_index]._extents[mesh] = extent
-            mesh.obstructions.append(self.obstructions[obst_index])
+            obst_tmp = Obstruction(obst_ordinal, side_surfaces, bound_indices, color_index,
+                                   block_type, texture_origin, rgba=rgba)
+            obst = next((o for o in self._merged_obstructions if o.id == obst_tmp.id), obst_tmp)
+
+            self.obstructions[mesh].append(obst)
+            # Holes will be handles differently
+            if obst_ordinal < 0:
+                if mesh not in obst._extents:
+                    obst._extents[mesh] = list()
+                obst._extents[mesh].append(extent)
+            else:
+                obst._extents[mesh] = extent
+                mesh.obstructions.append(obst)
+
+            if obst_ordinal < 0:
+                if obst_ordinal not in hole_extents:
+                    hole_extents[obst_ordinal] = list()
+                hole_extents[obst_ordinal].append(extent)
+
+        for ordinal, extents in hole_extents.items():
+            hole_extent = list()
+
+            minx = min(extents, key=attrgetter('x_start'))
+            hole_extent.append(minx.x_end)
+            maxx = max(extents, key=attrgetter('x_end'))
+            hole_extent.append(maxx.x_start)
+            miny = min(extents, key=attrgetter('y_start'))
+            hole_extent.append(miny.y_end)
+            maxy = max(extents, key=attrgetter('y_end'))
+            hole_extent.append(maxy.y_start)
+            minz = min(extents, key=attrgetter('z_start'))
+            hole_extent.append(minz.z_end)
+            maxz = max(extents, key=attrgetter('z_end'))
+            hole_extent.append(maxz.z_start)
+
+            hole_extent = Extent(*hole_extent)
+            obst = next(o for o in self._merged_obstructions if o.id == ordinal)
+            obst._extents[mesh].append(hole_extent)
+
+
+    @property
+    def _merged_obstructions(self) -> List[Obstruction]:
+        ret = list()
+        for _, obstructions in self.obstructions.items():
+            for obst in obstructions:
+                if not any(obst.id == o.id for o in ret):
+                    ret.append(obst)
+        return ret
 
     @log_error("vents")
     def _load_vents(self, smv_file: TextIO, mesh: Mesh):
@@ -323,7 +374,7 @@ class Simulation:
         for v in range(n):
             extent, vent_id, surface, texture_origin, circular_vent_origin, radius = temp_data[v]
             bound_indices, color_index, draw_type, rgba = read_common_info2()
-            if vent_id not in self.obstructions:
+            if vent_id not in self.ventilations:
                 self.ventilations[vent_id] = Ventilation(vent_id, surface, bound_indices,
                                                          color_index, draw_type, rgba=rgba,
                                                          texture_origin=texture_origin,
@@ -450,25 +501,27 @@ class Simulation:
                 extent, dimension = self._indices_to_extent(patch_info[:6], mesh)
                 orientation = patch_info[6]
                 obst_index = patch_info[7]
-                print(obst_index)
+
                 p = Patch(file_path, dimension, extent, orientation, cell_centered,
                           patch_offset + offset, n_t)
 
-                # Skip obstacles with ID 0, which just gives the extent of the (whole) mesh faces
+                # Skip obstacles with index 0, which just gives the extent of the (whole) mesh faces
                 # These might be needed in case of "closed" mesh faces
                 if obst_index != 0:
-                    obst_id = mesh.obstructions[obst_index - 1].id
-                    if obst_id not in patches:
-                        patches[obst_id] = list()
-                    patches[obst_id].append(p)
+                    obst_index -= 1  # Account for fortran indexing
+                    if obst_index not in patches:
+                        patches[obst_index] = list()
+                    patches[obst_index].append(p)
                 patch_offset += fdtype.new(
                     (('f', str(p.dimension.shape(cell_centered=False))),)).itemsize
 
-        for obst_id, p in patches.items():
+        for obst_index, p in patches.items():
             for patch in p:
                 patch._post_init(patch_offset)
-            self.obstructions[obst_id]._add_patches(bid, cell_centered, quantity, label, unit, mesh,
-                                                    p, times, n_t, lower_bounds, upper_bounds)
+
+            self.obstructions[mesh][obst_index]._add_patches(bid, cell_centered, quantity, label,
+                                                             unit, mesh, p, times, n_t,
+                                                             lower_bounds, upper_bounds)
 
     @log_error("pl3d")
     def _load_data_3d(self, smv_file: TextIO, line: str):
@@ -692,8 +745,7 @@ class Simulation:
         self.data_3d.clear_cache()
         self.isosurfaces.clear_cache()
         self.particles.clear_cache()
-        for obst in self.obstructions:
-            obst.clear_cache()
+        self.obstructions.clear_cache()
 
         if clear_persistent_cache:
             os.remove(Simulation._get_pickle_filename(self.root_path, self.chid))
